/***************************************************************************
 *   Copyright (C) 2004 by Reto Schoelly,,,                                *
 *   reto@reto-schoelly.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "defines.h"

#include "MainDialog.h"
#include "OpenGLWidget.h"
#include "setupgamedlg.h"

#include "mahjonggboard.h"
#include "configuregamedialog.h"
#include "manualdialog.h"
#include "hiscorelist.h"
#include "hiscorewidget.h"
#include "settingsbase.h"

#include "qlabel.h"
#include "qcombobox.h"
#include "qlineedit.h"
#include "qdialog.h"
#include "qcheckbox.h"
#include "qslider.h"

#include "mahjongggl.h"

#include "time.h"

#include "qmessagebox.h"
#include "qfiledialog.h"
#include "qfileinfo.h"
#include "qstring.h"
#include "qdir.h"
#include "qfileinfo.h"
#include "qpixmap.h"
#include "qiconview.h"

#include "gamedata_path.h"

extern int game_time;
extern char HISCORE_FILENAME[300];

void MainDialog :: NewGame()
{
	QString layout_string;
	QString tileset_string;
	QString background_string;

	SetupGameDlg dlg(this, 0);
	QString pathname;
	QString str = "";
	str = GAMEDATA_BASE_PATH;
	str += "/";

	pathname = str + "tilesets/";
	QDir tilesets_internal(pathname);

	pathname = str + "layouts/";
	QDir layouts_internal(pathname);

	pathname = str + "backgrounds/";
	QDir backgrounds_internal(pathname);

	tilesets_internal.setFilter(QDir::Files);
	tilesets_internal.setNameFilter("*.tileset");
	layouts_internal.setFilter(QDir::Files);
	layouts_internal.setNameFilter("*.layout");
	backgrounds_internal.setFilter(QDir::Files);
	backgrounds_internal.setNameFilter("*.bmp");

	const QFileInfoList *tileset_files;
	const QFileInfoList *layout_files;
	const QFileInfoList *background_files;

	tileset_files = tilesets_internal.entryInfoList();
	layout_files = layouts_internal.entryInfoList();
	background_files = backgrounds_internal.entryInfoList();

	QPixmap pm;
	QFileInfo *fi;
	QIconViewItem *item;

	QFileInfoListIterator it1(*tileset_files);
	while((fi = it1.current()) != 0)
	{
		++it1;
		pm.load(fi->dirPath() + "/" + fi->baseName() + "/default.bmp", 0, QPixmap::Auto);
		item = new QIconViewItem(dlg.iconView_tilesets, fi->baseName(), pm);
		item->setDragEnabled(false);
	}

	QFileInfoListIterator it2(*background_files);
	while((fi = it2.current()) != 0)
	{
		++it2;
		pm.load(fi->absFilePath(), 0, QPixmap::Auto);
		item = new QIconViewItem(dlg.iconView_backgrounds, fi->baseName(), pm);
		item->setDragEnabled(false);
	}

	QFileInfoListIterator it3(*layout_files);
	while((fi = it3.current()) != 0)
	{
		++it3;
		pm.load(fi->dirPath() + "/" + fi->baseName() + ".bmp", 0, QPixmap::Auto);
		item = new QIconViewItem(dlg.iconView_layouts, fi->baseName(), pm);
		item->setDragEnabled(false);
	}

	if(dlg.exec() == QDialog::Accepted)
	{
		it1.toFirst();
		it2.toFirst();
		it3.toFirst();

		if(dlg.selected_layout_item)
		{
			while((fi = it3.current()) != 0)
			{
				++it3;
				if(fi->baseName() == dlg.selected_layout_item->text())
				{
					layout_string = fi->absFilePath();
					break;
				}
			}
		}
		else
			layout_string = it3.current()->absFilePath();

		if(dlg.selected_background_item)
		{
			while((fi = it2.current()) != 0)
			{
				++it2;
				if(fi->baseName() == dlg.selected_background_item->text())
				{
					background_string = fi->absFilePath();
					break;
				}
			}
		}
		else
			background_string = it2.current()->absFilePath();

		if(dlg.selected_tileset_item)
		{
			while((fi = it1.current()) != 0)
			{
				++it1;
				if(fi->baseName() == dlg.selected_tileset_item->text())
				{
					tileset_string = fi->absFilePath();
					break;
				}
			}
		}
		else
			tileset_string = it1.current()->absFilePath();

		GLWidget->mgl->makeBoard(0, layout_string.latin1(), tileset_string.latin1(), background_string.latin1());
		textLabel_GameNr->setText(str.setNum(GLWidget->mgl->board->getRandomSeed()));
		GLWidget->updateGL();

		game_time = time(NULL);
	}
}

MainDialog :: MainDialog(QWidget *parent, const char *name)
{
	connect((QObject *)customMenuBar, SIGNAL(sendNewGame()), this, SLOT(NewGame()));
	connect((QObject *)customMenuBar, SIGNAL(sendConfigureGame()), this, SLOT(ConfigureGame()));
	connect((QObject *)customMenuBar, SIGNAL(sendReorderGame()), this, SLOT(ReorderGame()));
	connect((QObject *)customMenuBar, SIGNAL(sendUndo()), this, SLOT(Undo()));
	connect((QObject *)customMenuBar, SIGNAL(sendManual()), this, SLOT(Manual()));
	connect((QObject *)customMenuBar, SIGNAL(sendQuit()), this, SLOT(close()));
	connect((QObject *)customMenuBar, SIGNAL(sendHint()), this, SLOT(Hint()));
	connect((QObject *)customMenuBar, SIGNAL(sendSettings()), this, SLOT(Settings()));
	connect((QObject *)customMenuBar, SIGNAL(sendHiscores()), this, SLOT(Hiscores()));
	connect((QObject *)GLWidget, SIGNAL(sendUpdatePairs()), this, SLOT(UpdatePairs()));
	connect((QObject *)GLWidget, SIGNAL(sendRandomSeed(QString &)), this, SLOT(UpdateRandomSeed(QString &)));

	settings.rotation_type = ROTATION_DEFAULT;
	settings.mouse_speed_slider_pos = 50;
	settings.layout_pos = 0;
	settings.tileset_pos = 0;
	settings.background_pos = 0;
	settings.shadow = true;
}

MainDialog :: ~MainDialog()
{}

void MainDialog :: Hint()
{
	GLWidget->mgl->board->hint();
	GLWidget->updateGL();
}

void MainDialog :: Settings()
{
	SettingsBase dlg(this, 0);

	dlg.comboBoxRotation->setCurrentItem(settings.rotation_type);
	dlg.sliderMouseSpeed->setValue(settings.mouse_speed_slider_pos);
	dlg.checkBoxShadows->setChecked(settings.shadow);

	if(dlg.exec() == QDialog :: Accepted)
	{
		switch(dlg.comboBoxRotation->currentItem())
		{
		case 0:
			GLWidget->mgl->setRotationType(ROTATION_DEFAULT);
			settings.rotation_type = ROTATION_DEFAULT;
			break;
		case 1:
			GLWidget->mgl->setRotationType(ROTATION_ARCBALL);
			settings.rotation_type = ROTATION_ARCBALL;
			break;
		default:
			GLWidget->mgl->setRotationType(ROTATION_DEFAULT);
			settings.rotation_type = ROTATION_DEFAULT;
			break;
		}

		if(dlg.checkBoxShadows->isChecked())
		{
			GLWidget->mgl->shadow = true;
			settings.shadow = true;
		}
		else
		{
			GLWidget->mgl->shadow = false;
			settings.shadow = false;
		}


		GLWidget->mgl->setRotScale(dlg.sliderMouseSpeed->value());
		settings.mouse_speed_slider_pos = dlg.sliderMouseSpeed->value();

		GLWidget->updateGL();
	}
}

void MainDialog :: UpdateRandomSeed(QString &str)
{
	textLabel_GameNr->setText(str);
}

void MainDialog :: Hiscores()
{
	if(!HISCORE_FILENAME)
		return;

	QString str, number;
	HiscoreList list;
	if(list.readFile(HISCORE_FILENAME) == -1)
	{
		QMessageBox::information(this, "Could not load hiscore table", "No hiscores have been saved yet! Be the first :-))",QMessageBox::Ok, 0, 0);
		return;
	}
	list.sort();

	HiscoreWidget hiscore(this, 0);


	str = "";
	str.append(list.name[0]);
	number.setNum(list.score[0]);
	hiscore.lineEdit_name1->setText(str);
	hiscore.lineEdit_score1->setText(number);

	str = "";
	str.append(list.name[1]);
	number.setNum(list.score[1]);
	hiscore.lineEdit_name2->setText(str);
	hiscore.lineEdit_score2->setText(number);

	str = "";
	str.append(list.name[2]);
	number.setNum(list.score[2]);
	hiscore.lineEdit_name3->setText(str);
	hiscore.lineEdit_score3->setText(number);

	str = "";
	str.append(list.name[3]);
	number.setNum(list.score[3]);
	hiscore.lineEdit_name4->setText(str);
	hiscore.lineEdit_score4->setText(number);

	str = "";
	str.append(list.name[4]);
	number.setNum(list.score[4]);
	hiscore.lineEdit_name5->setText(str);
	hiscore.lineEdit_score5->setText(number);

	str = "";
	str.append(list.name[5]);
	number.setNum(list.score[5]);
	hiscore.lineEdit_name6->setText(str);
	hiscore.lineEdit_score6->setText(number);

	str = "";
	str.append(list.name[6]);
	number.setNum(list.score[6]);
	hiscore.lineEdit_name7->setText(str);
	hiscore.lineEdit_score7->setText(number);

	str = "";
	str.append(list.name[7]);
	number.setNum(list.score[7]);
	hiscore.lineEdit_name8->setText(str);
	hiscore.lineEdit_score8->setText(number);

	str = "";
	str.append(list.name[8]);
	number.setNum(list.score[8]);
	hiscore.lineEdit_name9->setText(str);
	hiscore.lineEdit_score9->setText(number);

	str = "";
	str.append(list.name[9]);
	number.setNum(list.score[9]);
	hiscore.lineEdit_name10->setText(str);
	hiscore.lineEdit_score10->setText(number);


	hiscore.exec();
}


void MainDialog :: ConfigureGame()
{
	QString str;

	ConfigureGameDialog *dlg = new ConfigureGameDialog(this, 0);
	dlg->exec();

	dlg->lineEdit_GameNum->text().toInt();
	//	GLWidget->mgl->makeBoard(dlg->lineEdit_GameNum->text().toInt(), "turtle.layout", "default.hires");

	textLabel_GameNr->setText(str.setNum(GLWidget->mgl->board->getRandomSeed()));
	GLWidget->updateGL();

	delete dlg;
}


void MainDialog :: ReorderGame()
{
	GLWidget->mgl->board->reorder();
	GLWidget->updateGL();
}


void MainDialog :: Undo()
{
	GLWidget->mgl->board->undo();
	GLWidget->updateGL();
}

void MainDialog :: Manual()
{
	ManualDialog dlg(this, 0);
	dlg.exec();
}


void MainDialog :: UpdatePairs()
{}


void MainDialog :: ToggleShadows()
{
	if(GLWidget->mgl->shadow)
		GLWidget->mgl->shadow = false;
	else
		GLWidget->mgl->shadow = true;
}
