/***************************************************************************
*   Copyright (C) 2004 by Reto Schoelly,,,                                *
*   reto@reto-schoelly.de                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "OpenGLWidget.h"

#include "math.h"

#include "defines.h"
#include "tile.h"
#include "mahjongggl.h"
#include "mahjonggboard.h"

#include "MainDialog.h"
#include "enternamedlg.h"
#include "qlineedit.h"
#include "qlabel.h"

#include "time.h"

#include "hiscorewidget.h"
#include "hiscorelist.h"

extern int game_time;
extern char HISCORE_FILENAME[300];


OpenGLWidget :: OpenGLWidget(QWidget *parent, const char *name) : QGLWidget(parent, name)
{
	mgl = new MahJonggGL();
}

OpenGLWidget :: ~OpenGLWidget()
{
    delete mgl;
}


void OpenGLWidget :: initializeGL()
{
	QString str;
	
	mgl->initializeGL();
}


void OpenGLWidget :: resizeGL(GLsizei width, GLsizei height)
{
	mgl->resizeGL(width, height);
}



void OpenGLWidget :: mouseMoveEvent(QMouseEvent *e)
{
    int x,y,button;
    x = e->x();
    y = e->y();
    if(e->button() == Qt::LeftButton)
        button = MOUSE_LEFT;
    if(e->button() == Qt::RightButton)
        button = MOUSE_RIGHT;
	   
	mgl->mouseMoveEvent(x,y,button);
	if(mgl->isMouseRightPressed()) updateGL();
}


void OpenGLWidget :: mousePressEvent(QMouseEvent *e)
{
    int x,y,button;
    x = e->x();
    y = e->y();
    if(e->button() == Qt::LeftButton)
        button = MOUSE_LEFT;
    if(e->button() == Qt::RightButton)
        button = MOUSE_RIGHT;
	   
	mgl->mousePressEvent(x,y,button);
	updateGL();
}


void OpenGLWidget :: mouseReleaseEvent(QMouseEvent *e)
{
    int x,y,button;
    x = e->x();
    y = e->y();
    if(e->button() == Qt::LeftButton)
        button = MOUSE_LEFT;
    if(e->button() == Qt::RightButton)
        button = MOUSE_RIGHT;
	   
 	mgl->mouseReleaseEvent(x,y,button);
}


// this method paints the scene and casts the shadows
void OpenGLWidget :: paintGL()
{
	mgl->paintGL();
	if(!mgl->board) return;
	int score;
	
	if((game_time != 0) && (mgl->board->getNumTilesOnBoard() == 0))
	{
		score = 10000 + (game_time - time(NULL));
		
		EnterNamedlg dlg(this, 0);
		game_time = 0;
		
		if(dlg.exec() == QDialog::Accepted)
		{
			HiscoreList list;
			list.readFile(HISCORE_FILENAME);
			list.sort();
			list.insertScore(score, dlg.lineEdit_PlayerName->text().latin1());
			list.sort();
			list.saveFile(HISCORE_FILENAME);
		
			QString str, number;
			
			HiscoreWidget hiscore(this, 0);
			
			
			str = "";
			str.append(list.name[0]);
			number.setNum(list.score[0]);
			hiscore.lineEdit_name1->setText(str);
			hiscore.lineEdit_score1->setText(number);
			
			str = "";
			str.append(list.name[1]);
			number.setNum(list.score[1]);
			hiscore.lineEdit_name2->setText(str);
			hiscore.lineEdit_score2->setText(number);
			
			str = "";
			str.append(list.name[2]);
			number.setNum(list.score[2]);
			hiscore.lineEdit_name3->setText(str);
			hiscore.lineEdit_score3->setText(number);
			
			str = "";
			str.append(list.name[3]);
			number.setNum(list.score[3]);
			hiscore.lineEdit_name4->setText(str);
			hiscore.lineEdit_score4->setText(number);
			
			str = "";
			str.append(list.name[4]);
			number.setNum(list.score[4]);
			hiscore.lineEdit_name5->setText(str);
			hiscore.lineEdit_score5->setText(number);
			
			str = "";
			str.append(list.name[5]);
			number.setNum(list.score[5]);
			hiscore.lineEdit_name6->setText(str);
			hiscore.lineEdit_score6->setText(number);
			
			str = "";
			str.append(list.name[6]);
			number.setNum(list.score[6]);
			hiscore.lineEdit_name7->setText(str);
			hiscore.lineEdit_score7->setText(number);
				
			str = "";
			str.append(list.name[7]);
			number.setNum(list.score[7]);
			hiscore.lineEdit_name8->setText(str);
			hiscore.lineEdit_score8->setText(number);
			
			str = "";
			str.append(list.name[8]);
			number.setNum(list.score[8]);
			hiscore.lineEdit_name9->setText(str);
			hiscore.lineEdit_score9->setText(number);
			
			str = "";
			str.append(list.name[9]);
			number.setNum(list.score[9]);
			hiscore.lineEdit_name10->setText(str);
			hiscore.lineEdit_score10->setText(number);
			
			
			hiscore.exec();
		}
	}
}


