/***************************************************************************
*   Copyright (C) 2004 by Reto Schoelly,,,                                *
*   reto@reto-schoelly.de                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#include "cbitmapfile.h"

#include "malloc.h"
#include "stdio.h"

CBitmapFile::CBitmapFile()
{
	image = NULL;
}


CBitmapFile::~CBitmapFile()
{
	if(image)
	{
		if(image->data) delete image->data;
		delete image;
	}
}


int CBitmapFile::LoadBMP(char *filename)
{
	FILE *file;
	unsigned int size;                  // Size of the image in bytes
	unsigned long i;                    
	unsigned short int planes;          // Number of planes in bitmap
	unsigned short int bpp;             // Num of bits per pixel
	char temp;
	
	if(image)
	{
		if(image->data) delete image->data;
		delete image;
	}
	image = new Image;
	
	// Open the file
	if ((file = fopen(filename, "rb"))==NULL)
	{
		printf("File %s not found\n",filename);
		return ERROR_FILENOTFOUND;
	}
	
	// Read width and height of the bitmap. This information begins at byte 18 - go there
	fseek(file, 18, SEEK_CUR);
	
	// Read the width
	if ((i = fread(&image->sizeX, 4, 1, file)) != 1)
	{
		printf("Error reading width from %s.\n", filename);
		return ERROR_READINGFILEFAILED;
	}
	
	// Read the height
	if ((i = fread(&image->sizeY, 4, 1, file)) != 1)
	{
		printf("Error reading height from %s.\n", filename);
		return ERROR_READINGFILEFAILED;
	}
	
	// Calculate the size, three bytes per pixel (24bit colors needed!)
	size = image->sizeX * image->sizeY * 3;
	
	// Read the planes
	if ((fread(&planes, 2, 1, file)) != 1)
	{
		printf("Error reading planes from %s.\n", filename);
		return ERROR_READINGFILEFAILED;
	}
	if (planes != 1)
	{
		printf("Planes from %s is not 1: %u\n", filename, planes);
		return ERROR_READINGFILEFAILED;
	}
	
	// Read the bytes per pixel. Here, only 24bit color bitmaps are used.
	if ((i = fread(&bpp, 2, 1, file)) != 1)
	{
		printf("Error reading bpp from %s.\n", filename);
		return ERROR_READINGFILEFAILED;
	}
	if (bpp != 24)
	{
		printf("Bpp from %s is not 24: %u\n", filename, bpp);
		return ERROR_BPP;
	}
	
	// Seek past the rest of the bitmap header to the beginning of the image data
	fseek(file, 24, SEEK_CUR);
	
	// Read the data
	image->data = (char *) malloc(size);
	if (image->data == NULL)
	{
		printf("Error allocating memory for color corrected image data");
		return ERROR_MEMORY;
	}
	
	if ((i = fread(image->data, size, 1, file)) != 1)
	{
		printf("Error reading image data from %s.\n", filename);
		return ERROR_READINGFILEFAILED;
	}
	
	// now reverse the colors (they are stored as bgr, convert to rgb)
	for (i=0;i<size;i+=3)
	{
		temp = image->data[i];
		image->data[i] = image->data[i+2];
		image->data[i+2] = temp;
	}
	
	fclose(file);
	
    return FILE_OK;
}

