/***************************************************************************
 *   Copyright (C) 2004 by Reto Schoelly,,,                                *
 *   reto@reto-schoelly.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "environmentgl.h"
#include "cbitmapfile.h"

#include "malloc.h"
#include "stdlib.h"
#include "stdio.h"
#include "string.h"
#include "stdarg.h"

#include "GL/gl.h"

#define ENV_STR_LEN 50

void EnvironmentGL::helper(char* str, const char* fmt, ...)
{
	va_list args;
	va_start(args, fmt);
	vsprintf(str, fmt, args);
	va_end(args);
}

EnvironmentGL::EnvironmentGL(const char *filename)
{
    face_list_head = NULL;
    env_name = NULL;
    
    int n, num_of_face;
    
    face *list = NULL;
    triangle *tri;
    
    char *str = (char *)malloc(ENV_STR_LEN);
    env_name = (char *)malloc(100);
    
    memset(str, ENV_STR_LEN, '\0');
    memset(env_name, 100, '\0');

    FILE *file = fopen(filename, "r");
    
    while(strcmp(str, "END_OF_FILE") != 0)
    {
	memset(str, ENV_STR_LEN, '\0');
	fgets(str, ENV_STR_LEN, file);
	
	if(strcmp(str, "DEF"))
	{
		memset(str, ENV_STR_LEN, '\0');
		fgets(str, ENV_STR_LEN, file);
		if(strcmp(str, "ENV_NAME") == 0)
		{
			fgets(env_name, ENV_STR_LEN, file);
			fgets(str, ENV_STR_LEN, file);
			fgets(str, ENV_STR_LEN, file);
		}
		else if(strcmp(str, "NUMBER_OF_FACES") == 0)
		{
			fgets(env_name, ENV_STR_LEN, file);
			fgets(str, ENV_STR_LEN, file);
			memset(str, ENV_STR_LEN, '\0');
			helper(str, "%d", num_faces);
			fgets(str, ENV_STR_LEN, file);
			
			for(n = 0; n < num_faces; n++)
			{
				if(list == NULL)
				{
					face_list_head = (face *)malloc(sizeof(face));
					face_list_head->next = NULL;
					list = face_list_head;
				}
				else
				{
					list->next = (face *)malloc(sizeof(face));
					list->next->next = NULL;
					list = list->next;
				}
			}
		}
		else if(strcmp(str, "FACE") == 0)
		{
			memset(str, ENV_STR_LEN, '\0');
			fgets(str, ENV_STR_LEN, file);
			helper(str, "%d", num_of_face);
			
			list = face_list_head;
			
			for(n = 1; n < num_of_face; n++)
			{
				list = list->next;
			}
			
			memset(str, ENV_STR_LEN, '\0');
			fgets(str, ENV_STR_LEN, file);
			helper(str, "%d", list->num_triangles);
			list->triangles = (triangle *)malloc(sizeof(triangle)*list->num_triangles);
			
			list->texture_filename = (char *)malloc(ENV_STR_LEN);
			memset(list->texture_filename, ENV_STR_LEN, '\0');
			fgets(list->texture_filename, ENV_STR_LEN, file);
			list->texture_filename[strlen(list->texture_filename)-1] = '\0';
			
			memset(str, ENV_STR_LEN, '\0');
			fgets(str, ENV_STR_LEN, file);
			helper(str, "%d", list->texture_width);
			memset(str, ENV_STR_LEN, '\0');
			fgets(str, ENV_STR_LEN, file);
			helper(str, "%d",  list->texture_height);
			
			fgets(str, ENV_STR_LEN, file);
			
			tri = list->triangles;
			for(n = 0; n < list->num_triangles; n++)
			{
				memset(str, ENV_STR_LEN, '\0');
				fgets(str, ENV_STR_LEN, file);
				helper(str, "%f", tri->vertices[0]);
				memset(str, ENV_STR_LEN, '\0');
				fgets(str, ENV_STR_LEN, file);
				helper(str, "%f", tri->vertices[1]);
				memset(str, ENV_STR_LEN, '\0');
				fgets(str, ENV_STR_LEN, file);
				helper(str, "%f", tri->vertices[2]);
				
				fgets(str, ENV_STR_LEN, file);
				
				memset(str, ENV_STR_LEN, '\0');
				fgets(str, ENV_STR_LEN, file);
				helper(str, "%f", tri->normal[0]);
				memset(str, ENV_STR_LEN, '\0');
				fgets(str, ENV_STR_LEN, file);
				helper(str, "%f", tri->normal[1]);
				memset(str, ENV_STR_LEN, '\0');
				fgets(str, ENV_STR_LEN, file);
				helper(str, "%f", tri->normal[2]);
			
				fgets(str, ENV_STR_LEN, file);
				
				tri++;
			}
		}
	}
    }
    
    fclose(file);
    free(str);
}


EnvironmentGL::~EnvironmentGL()
{
	face *f = face_list_head;
	face *tmp = NULL;
	
	free(env_name);
	
	while(f)
	{
		tmp = f->next;
		if(f->triangles) free(f->triangles);
		if(f->texture_filename) free(f->texture_filename);
		if(f->texture) delete f->texture;
		free(f);
		f = tmp;
	}
}




/*!
    \fn EnvironmentGL::initializeEnvironment()
 */
bool EnvironmentGL::initializeEnvironment()
{
	face *list = face_list_head;
	
	while(list)
	{
		
		list->texture = new CBitmapFile();
		list->texture->LoadBMP(list->texture_filename);
		
		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		glGenTextures(1, &list->texture_name);
		glBindTexture(GL_TEXTURE_2D, list->texture_name);
		
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, list->texture_width, list->texture_height, 0, GL_RGB, GL_UNSIGNED_BYTE, list->texture->image->data);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		
		list = list->next;
	}
	
	return true;
}


/*!
    \fn EnvironmentGL::draw()
 */
void EnvironmentGL::draw()
{
	face *list = face_list_head;
	
	while(list)
	{
		glBindTexture(GL_TEXTURE_2D, list->texture_name);
		
		glBegin(GL_TRIANGLES);
		// TODO: implement drawing routines
		glEnd();
		
		list = list->next;
	}
}
