/***************************************************************************
 *   Copyright (C) 2004 by Reto Schoelly,,,                                *
 *   reto@reto-schoelly.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MAHJONGGBOARD_H
#define MAHJONGGBOARD_H

/**
@author Reto Schoelly,,,
*/

#include "defines.h"

class Tile;
class TileGL;


class MahJonggBoard
{
public:
	MahJonggBoard();
	~MahJonggBoard();

	bool isBlocked(int x, int y, int z);
	bool selectTile(int x, int y, int z);
	bool generateSolvableBoard(int seed, const char *layout);

	unsigned int getDisplayList(int x, int y, int z);
	unsigned int getTextureName(int x, int y, int z);

	bool getBoardGeometry(int *width, int *height, int *depth);
	Tile * getTile(int x, int y, int z);
	int getNumTilesOnBoard();

	int calcNumRemovablePairs();

	bool isInitialized();

	int convertCoordinates(int x, int y, int z);
	bool loadLayout(const char *layout_filename);

	bool undo();
	bool reorder();

	int getRandomSeed();

	int hint();

private:
	bool findBoundaryTilesInColumn(int x, int y, int z, int *left, int *right, char *column_used, char *column_full);

	bool tilesMatch(Tile *tile1, Tile *tile2);
	bool removeTile(int num);

	int generateTileType();
	bool generateUsablePositions();

	void setPositionUsable(int pos);
	void setPositionUnusable(int x, int y, int z);
	int setTile(int pos, int tile_type, int state);
	bool isUsable(int x, int y, int z);
	bool usabilityTest(int x, int y, int z);

	bool resolveUsablePositions();

	bool flipHoriz(int x, int y, int z);
	bool flipVert(int x, int y, int z);
	bool flipDepth(int x, int y, int z);

	int random_seed;

	int num_tiles_left;
	int numtiles;
	bool initialized;
	int removable_tiles_list[BOARD_SIZE];

	int board_width;
	int board_height;
	int board_depth;

	int selected_tile_1;
	int selected_tile_2;

	int num_tiles_on_board;
	int num_usable_positions;

	int hint_pos1;
	int hint_pos2;

	int undo_list[BOARD_SIZE][2];
	int num_turns;

	int tileset[NUM_TILES];

	Tile *field[BOARD_SIZE];
	int usable_positions[BOARD_SIZE];
};

#endif
