/***************************************************************************
 *   Copyright (C) 2004 by Reto Schoelly,,,                                *
 *   reto@reto-schoelly.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MAHJONGGGL_H
#define MAHJONGGGL_H

/**
@author Reto Schoelly,,,
*/

class MahJonggBoard;
class CBitmapFile;
class TileGL;

#include "defines.h"



class MahJonggGL
{
public:

public:
	MahJonggGL();
	~MahJonggGL();

	void initializeGL();
	void resizeGL(int width, int height);
	void paintGL();

	void mouseMoveEvent(int x, int y, int button);
	void mousePressEvent(int x, int y, int button);
	void mouseReleaseEvent(int x, int y, int button);
	void setupCamera();

	void RenderSelectionScene(int layer, bool shadow);
	void select(int name);

	unsigned int createBackground(const char *tex_filename);
	bool makeBoard(int random_seed, const char *layout, const char *tileset, const char *background);

	// this creates a matrix for shadow casting
	void CreateShadowMatrix(float points[3][3], float light[3], float matrix[4][4]);

	bool isMouseRightPressed();

	bool setRotationType(int type);

	void enableGL(bool enable);
	bool setRotScale(int val);

	MahJonggBoard *board;

	bool shadow;

	float Omni0Color[4];
	float Omni0Amb[4];
	float Omni0Pos[4];

private:
	bool enabled;
	int rotation_type;
	float ROT_SCALE;

	bool createTileGLs(const char *tileset_graphics_filename);

	// this creates the display lists for the digits displaying the number of remaining pairs
	void CreateNumberDisplayLists(char *directory);
	void DisplayCongratulations(char *directory);

	void drawBackground();

	void mouseRotationDefault(int x, int y);
	void mouseRotationArcBall(int x, int y);

	// ArcBall rotation contributed by Michael George
	//{
	void mapToSphere( int x, int y, float result[3] );
	float drag_start[3];
	float axis[3], cosangle;
	float transform[16];
	//}

	unsigned int background_texture;
	unsigned int bg_width;
	unsigned int bg_height;
	CBitmapFile *bg_tex;

	TileGL *tiles[NUM_TEXTURES];
	TileGL *shadow_tile;

	// these are the 10 bitmaps for the digits
	CBitmapFile *number_bmp[10];
	unsigned int number_texnum[10]; // the corresponding texture numbers

	CBitmapFile *congratulations;
	unsigned int congratulations_texnum;

	float camera_target_x;
	float camera_target_y;
	float camera_target_z;

	int board_width;
	int board_height;
	int board_depth;

	bool pick;

	bool show_removable;

	int num_pairs;

	float cam_x;
	float cam_y;
	float cam_z;

	int window_size_x;
	int window_size_y;

	bool mb_r;

	int mouse_x;
	int mouse_y;

	float angle_x;
	float angle_z;

	bool stencil;
};

#endif
