/***************************************************************************
 *   Copyright (C) 2004 by Reto Schoelly,,,                                *
 *   reto@reto-schoelly.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "tilegl.h"

#include "defines.h"
#include "cbitmapfile.h"

#include "math.h"
#include "malloc.h"
#include "string.h"

// this directive is only needed for windows, so if
// you are compiling with linux, enter a "#define LINUX" in defines.h
#ifndef LINUX
#include "..\workspace\MahJongg3D\StdAfx.h"
#endif

#include "GL/gl.h"


TileGL::TileGL(int fillet_resolution, int texture_resolution, const char *texture_filename)
{
	display_list_unmarked = 0;
	display_list_marked = 0;
	texture_name = 0;

	fillet_res = fillet_resolution;
	texture_res = texture_resolution;
	texture_file = strdup(texture_filename);

	bmfile = new CBitmapFile();
}


TileGL::~TileGL()
{
	delete bmfile;
	free(texture_file);
}

// initialize the tile and load the textures
bool TileGL::initializeTile(bool shadow)
{
	int counter, list;
	float x, y, z, angle, angle2;
	float dx, dy, dz, dx2, dy2, dz2;
	float nx,ny,nz;
	float nx2,ny2,nz2;
	int count;
	int TILE_FILLET_STEPS = fillet_res;

	GLfloat Diffuse[4] = {0.8f, 0.8f, 0.8f, 1.0f};
	GLfloat Diffuse_marked[4] = {1.0f, 1.0f, 2.3f, 1.0f};
	GLfloat Specular[4] = {0.9f, 0.9f, 0.9f, 1.0f};
	GLfloat Specular_marked[4] = {0.1f, 0.1f, 0.8f, 0.4f};
	GLfloat Emission[4] = {0.1f, 0.1f, 0.1f, 1.0f};
	GLfloat Emission_marked[4] = {0.1f, 0.1f, 0.8f, 0.4f};
	GLfloat Shininess = 120.0f;
	GLfloat Shininess_marked = 0.5f;

	bmfile->LoadBMP(texture_file);

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glGenTextures(1, &texture_name);
	glBindTexture(GL_TEXTURE_2D, texture_name);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, texture_res, texture_res, 0, GL_RGB, GL_UNSIGNED_BYTE, bmfile->image->data);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	for(counter = 0; counter < 2; counter++)
	{
		if(counter == 0)
			list = display_list_unmarked = glGenLists(1);
		else
			list = display_list_marked = glGenLists(1);
		glNewList(list, GL_COMPILE);

		glColor4f(1.0f,1.0f,1.0f,0.35f);
		
		glBegin(GL_QUADS);

		if(!shadow)
		{
			if(counter == 0)
			{
				glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, Diffuse);
				glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, Specular);
				glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, &Shininess);
				glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, Emission);
			}
			else
			{
				glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, Diffuse_marked);
				glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, Specular_marked);
				glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, &Shininess_marked);
				glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, Emission_marked);
			}
		}
		else
		
		glColor4f(1.0f,1.0f,1.0f,0.35f);


		glNormal3f(0.0f, 0.0f, -1.0f);
		glVertex3f(0.0f,	0.0f,		0.0f);
		glVertex3f(TILE_WIDTH,	0.0f,		0.0f);
		glVertex3f(TILE_WIDTH,	TILE_HEIGHT,	0.0f);
		glVertex3f(0.0f,	TILE_HEIGHT,	0.0f);

		glNormal3f(0.0f, 0.0f, 1.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(TILE_FILLET_RADIUS,	TILE_FILLET_RADIUS,		TILE_DEPTH);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(TILE_WIDTH - TILE_FILLET_RADIUS,	TILE_FILLET_RADIUS,		TILE_DEPTH);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(TILE_WIDTH - TILE_FILLET_RADIUS,	TILE_HEIGHT - TILE_FILLET_RADIUS,	TILE_DEPTH);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(TILE_FILLET_RADIUS,	TILE_HEIGHT - TILE_FILLET_RADIUS,	TILE_DEPTH);

		glNormal3f(-1.0f, 0.0f, 0.0f);
		glVertex3f(0.0f,	0.0f,		0.0f);
		glVertex3f(0.0f,	0.0f,		TILE_DEPTH-TILE_FILLET_RADIUS);
		glVertex3f(0.0f,	TILE_HEIGHT,	TILE_DEPTH-TILE_FILLET_RADIUS);
		glVertex3f(0.0f,	TILE_HEIGHT,	0.0f);

		glNormal3f(1.0f, 0.0f, 0.0f);
		glVertex3f(TILE_WIDTH,	TILE_HEIGHT,	0.0f);
		glVertex3f(TILE_WIDTH,	TILE_HEIGHT,	TILE_DEPTH-TILE_FILLET_RADIUS);
		glVertex3f(TILE_WIDTH,	0.0f,		TILE_DEPTH-TILE_FILLET_RADIUS);
		glVertex3f(TILE_WIDTH,	0.0f,		0.0f);

		glNormal3f(0.0f, 1.0f, 0.0f);
		glVertex3f(0.0f,	TILE_HEIGHT,	TILE_DEPTH-TILE_FILLET_RADIUS);
		glVertex3f(TILE_WIDTH,	TILE_HEIGHT,	TILE_DEPTH-TILE_FILLET_RADIUS);
		glVertex3f(TILE_WIDTH,	TILE_HEIGHT,	0.0f);
		glVertex3f(0.0f,	TILE_HEIGHT,	0.0f);

		glNormal3f(0.0f, -1.0f, 0.0f);
		glVertex3f(0.0f,	0.0f,		0.0f);
		glVertex3f(TILE_WIDTH,	0.0f,		0.0f);
		glVertex3f(TILE_WIDTH,	0.0f,		TILE_DEPTH-TILE_FILLET_RADIUS);
		glVertex3f(0.0f,	0.0f,		TILE_DEPTH-TILE_FILLET_RADIUS);

		glEnd();

		glBegin(GL_QUADS);

		x = TILE_FILLET_RADIUS;
		y = TILE_FILLET_RADIUS;
		z = TILE_DEPTH-TILE_FILLET_RADIUS;

		for(count = 0; count < TILE_FILLET_STEPS; count++)
		{
			angle = (float)count / TILE_FILLET_STEPS * PI * 0.5f;
			angle2 = (float)(count+1) / TILE_FILLET_STEPS * PI * 0.5f;

			dx = -TILE_FILLET_RADIUS * (float)cos(angle);
			dy = -TILE_FILLET_RADIUS * (float)cos(angle);
			dz = TILE_FILLET_RADIUS * (float)sin(angle);
			dx2 = -TILE_FILLET_RADIUS * (float)cos(angle2);
			dy2 = -TILE_FILLET_RADIUS * (float)cos(angle2);
			dz2 = TILE_FILLET_RADIUS * (float)sin(angle2);

			nx = 0;
			ny = -(float)cos(angle);
			nz = (float)sin(angle);
			nx2 = 0;
			ny2 = -(float)cos(angle2);
			nz2 = (float)sin(angle2);

			glNormal3f(nx, ny, nz);
			glVertex3f(x+dx, y+dy, z+dz);
			glVertex3f(TILE_WIDTH-x-dx, y+dy, z+dz);
			glNormal3f(nx2, ny2, nz2);
			glVertex3f(TILE_WIDTH-x-dx2, y+dy2, z+dz2);
			glVertex3f(x+dx2, y+dy2, z+dz2);

			nx = -(float)cos(angle);
			ny = 0;
			nz = (float)sin(angle);
			nx2 = -(float)cos(angle2);
			ny2 = 0;
			nz2 = (float)sin(angle2);
			glNormal3f(nx2, ny2, nz2);
			glVertex3f(x+dx2, y+dy2, z+dz2);
			glVertex3f(x+dx2, TILE_HEIGHT-y-dy2, z+dz2);
			glNormal3f(nx, ny, nz);
			glVertex3f(x+dx, TILE_HEIGHT-y-dy, z+dz);
			glVertex3f(x+dx, y+dy, z+dz);

			nx = 0;
			ny = (float)cos(angle);
			nz = (float)sin(angle);
			nx2 = 0;
			ny2 = (float)cos(angle2);
			nz2 = (float)sin(angle2);
			glNormal3f(nx2, ny2, nz2);
			glVertex3f(x+dx2, TILE_HEIGHT-y-dy2, z+dz2);
			glVertex3f(TILE_WIDTH-x-dx2, TILE_HEIGHT-y-dy2, z+dz2);
			glNormal3f(nx, ny, nz);
			glVertex3f(TILE_WIDTH-x-dx, TILE_HEIGHT-y-dy, z+dz);
			glVertex3f(x+dx, TILE_HEIGHT-y-dy, z+dz);

			nx = (float)cos(angle);
			ny = 0;
			nz = (float)sin(angle);
			nx2 = (float)cos(angle2);
			ny2 = 0;
			nz2 = (float)sin(angle2);
			glNormal3f(nx, ny, nz);
			glVertex3f(TILE_WIDTH-x-dx, y+dy, z+dz);
			glVertex3f(TILE_WIDTH-x-dx, TILE_HEIGHT-y-dy, z+dz);
			glNormal3f(nx2, ny2, nz2);
			glVertex3f(TILE_WIDTH-x-dx2, TILE_HEIGHT-y-dy2, z+dz2);
			glVertex3f(TILE_WIDTH-x-dx2, y+dy2, z+dz2);
		}

		glEnd();
		glEndList();
	}

	return true;
}


unsigned int TileGL::getDisplayListUnmarked()
{
	return display_list_unmarked;
}

unsigned int TileGL::getDisplayListMarked()
{
	return display_list_marked;
}


unsigned int TileGL::getTextureName()
{
	return texture_name;
}

